//+------------------------------------------------------------------+
//| Chances_robot.mq4 |
//| Copyright 2009, MetaQuotes Software Corp. |
//| http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright 2009, MetaQuotes Software Corp."
#property link "http://www.metaquotes.net"

extern string _si_E = "Parameters ------------------";
extern int _D=5; // % - the minimal distance between pending orders (% the current price channel, 32 by default)
extern double _Init_Init_Lot=0.01; // The initial lot (by default  set it to win/lose 1% of the depo in case of TP/SL);
extern double _Max_Init_Lot=5; //Maximum size of the single deals position. In case its reached, Lot is reset to Init_Lot
extern int _Min_Channel=60; // pips  the minimal price channel width to set the pending orders. In case the channel is narrower  wait
extern int _Max_Channel=230; // pips  the maximal channel to set the orders. In case the channel is wider, the opposite orders  Stop-loss and Take-profit are set at Max_Channels distance, not at the channels border
extern bool _Double_First_Lot=True; // False means we do not double the first revert order. This approach reduces the risk of reaching Max_Lot and corresponding losses
extern int _ChannelPeriod=55; // Price channel depth
extern int _Max_orders_count = 1000;

extern string _si_Interception = "Expert control -------------------";
extern bool _bInterception=false; // Intercept the expert's start
extern int _TimeSleep=1; // Time to sleep to save the PC resources
int g_FunctionExpert=0; // Expert activity flag

double mult, Min_Channel, Max_Channel, Channel, High_border, Low_border, New_border, StopLevel, Range;
double rLot;
int n, err, rOp, Ticket, pool_size = 1000000;

//+------------------------------------------------------------------+
//| expert initialization function |
//+------------------------------------------------------------------+
int init()
{
   if(Digits > 3)
      mult = 0.0001;  // EUR - #.####(#)
   else
      mult = 0.01;    // JPY - #.##(#)
   
   if(MarketInfo(Symbol(), MODE_MAXLOT) < _Max_Init_Lot)
      _Max_Init_Lot = MarketInfo(Symbol(), MODE_MAXLOT);

   Min_Channel = NormalizeDouble(_Min_Channel * mult, Digits);
   Max_Channel = NormalizeDouble(_Max_Channel * mult, Digits);
   StopLevel = MarketInfo(Symbol(),MODE_STOPLEVEL) * Point + MarketInfo(Symbol(),MODE_SPREAD) * Point;
}
//+------------------------------------------------------------------+
//| expert deinitialization function |
//+------------------------------------------------------------------+
int deinit()
{
}
//+------------------------------------------------------------------+
//| expert start function |
//+------------------------------------------------------------------+
int start()
{
  // Until the user breaks the execution - loop
  while(!IsStopped() && IsTradeAllowed())
  {
    if(!IsTesting() && !MarketInfo(Symbol(),MODE_TRADEALLOWED))
    {
      Comment("Trade on instrument prohibited! ");
      return;
    }

    if(!IsTesting()
        && (
        (!IsExpertEnabled()
        || !IsConnected()
        || IsTradeContextBusy()
        )))
      break;
    else
      g_FunctionExpert=2;

    RefreshRates();

    Open_orders(54, pool_size);

    Process_orders();

    if(IsTesting()
        || !_bInterception
        || g_FunctionExpert!=2)
      break;

    Sleep(_TimeSleep);
  }
}
// ////////////////////////////////////////////////////////////////////////////
//    OPEN ORDERS
// ////////////////////////////////////////////////////////////////////////////
void Open_orders(int N, int C)
{
// Check if we've got too many orders opened already
      if(OrdersTotal() >= _Max_orders_count / 2)
         return;

// Get the parameters of our indicator
      High_border=iCustom(Symbol(),Period(),"PriceChannel",N,0,0); // _ChannelPeriod
      Low_border=iCustom(Symbol(),Period(),"PriceChannel",N,1,0);
      Channel=NormalizeDouble(High_border-Low_border,Digits);
      Range = Channel / 100 * _D + MarketInfo(Symbol(),MODE_SPREAD) * Point;
      
// Check if the Channel is wide enough
      if(Channel < Min_Channel)
         return;
         
// Limit the Channel
      if(Channel > Max_Channel)
         Channel = Max_Channel;
       
//  Check if we have a bounce from the High border, which allows us to place the Order
      if((Ask < High_border - StopLevel)
            && (High_border == iCustom(Symbol(),Period(),"PriceChannel",N,0,3)))
      {
//  and there are no BUY Orders within _D range near High border
         if(No_Orders_near(OP_BUYSTOP, High_border))
         {
//   System 0 (default): Place opening order followed by Stop-loss and Take-profit
                if(0 > OrderSend(Symbol(), OP_BUYSTOP, _Init_Init_Lot, High_border
                              , 0 // Slippage
                              , NormalizeDouble(High_border - Channel, Digits) // Stop loss
                              , NormalizeDouble(High_border + Channel, Digits) // Take profit
                              , "" // Comment
                              , C // Magic
                              , 0 // Life-Time
                              , Blue))
                {
                  err=GetLastError();
                  Print("error(",err,"): ", ErrorDescription(err));
                }
                else
                {
//    Delete all active BUY orders above
                  for(n=0; n<OrdersTotal(); n++)
                  {
                      if(!OrderSelect(n,SELECT_BY_POS,MODE_TRADES))
                        continue;

                      if((OrderMagicNumber() < C) || (OrderMagicNumber() >= C + pool_size))
                        continue;

                      if((OrderSymbol() == Symbol()) && (OrderMagicNumber() % pool_size == 0)
                              && ((OrderType() == OP_BUYSTOP) || (OrderType() == OP_SELLLIMIT)))
                         if((OrderOpenPrice() > High_border)) // && (OrderOpenPrice() < High_border + Range))
                            OrderDelete(OrderTicket(), Red);
                  }
                }
//  System 2: (another channel upon the Channel):
//  This is to catch the spring and re-open the position
                New_border = NormalizeDouble(High_border + Channel / 100 * _D / 2, Digits);
                if(0 > OrderSend(Symbol(), OP_SELLLIMIT, _Init_Init_Lot, New_border
                              , 0 // Slippage
                              , NormalizeDouble(New_border + Channel, Digits) // Stop loss
                              , NormalizeDouble(New_border - Channel, Digits) // Take profit
                              , "" // Comment
                              , C // Magic
                              , 0 // Life-Time
                              , Blue))
                {
                  err=GetLastError();
                  Print("error (Syst 2) (",err,"): ", ErrorDescription(err));
                }
         }
      }

//  Check if we have a bounce from the Low border, which allows us to place the Order
      if((Bid > Low_border + StopLevel)
            && (Low_border == iCustom(Symbol(),Period(),"PriceChannel",N,0,3)))
      {
//  and there are no BUY Orders within _D range near High border (including Reverts)
         if(No_Orders_near(OP_SELLSTOP, Low_border))
         {
//   System 0 (default): Place opening order followed by Stop-loss and Take-profit
                Ticket = OrderSend(Symbol(), OP_SELLSTOP, _Init_Init_Lot, Low_border
                              , 0 // Slippage
                              , NormalizeDouble(Low_border + Channel, Digits) // Stop loss
                              , NormalizeDouble(Low_border - Channel, Digits) // Take profit
                              , "" // Comment
                              , C // Magic
                              , 0 // Life-Time
                              , Blue);

                if(Ticket < 0)
                {
                  err=GetLastError();
                  Print("error(",err,"): ", ErrorDescription(err));
                }
                else
                {
//    Delete all active orders below (of the same Channel only)
                  for(n=0; n<OrdersTotal(); n++)
                  {
                      if(!OrderSelect(n,SELECT_BY_POS,MODE_TRADES))
                        continue;

                      if((OrderMagicNumber() < C) || (OrderMagicNumber() >= C + pool_size))
                        continue;

                      if((OrderSymbol() == Symbol()) && (OrderMagicNumber() % pool_size == 0) 
                              && ((OrderType() == OP_SELLSTOP) || (OrderType() == OP_BUYLIMIT)))
                         if((OrderOpenPrice() < Low_border)) // && (OrderOpenPrice() > Low_border - Range))
                            OrderDelete(OrderTicket(), Red);
                  }
                }
//  System 2: (another channel below the channel):
//  This is to catch the spring and re-open the position
                New_border = NormalizeDouble(Low_border - Channel / 100 * _D / 2, Digits);
                if(0 > OrderSend(Symbol(), OP_BUYLIMIT, _Init_Init_Lot, New_border
                              , 0 // Slippage
                              , NormalizeDouble(New_border - Channel, Digits) // Stop loss
                              , NormalizeDouble(New_border + Channel, Digits) // Take profit
                              , "" // Comment
                              , C // Magic
                              , 0 // Life-Time
                              , Blue))
                {
                  err=GetLastError();
                  Print("error (Syst 2) (",err,"): ", ErrorDescription(err));
                }
         }
      }
}
// ////////////////////////////////////////////////////////////////////////////
//    PROCESS ORDERS
// ////////////////////////////////////////////////////////////////////////////
int Process_orders()
{
// Select those Reverts, that have their Revertees closed with profit
//  and those orders that were triggered but not reverted yet

/*     int revert = 0;
     double Price = 0;
     color Color;
     for(int i=0; i<OrdersTotal(); i++)
     {
        if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
              continue;
        Price = Price + OrderProfit();
        
        if((OrderLots()>_Init_Init_Lot) && (OrderType()<=1))
        {
            Price = Price - (OrderLots()-_Init_Init_Lot)*MathAbs(OrderOpenPrice()-OrderStopLoss())*100000;
            revert = 1;
//            Print("lost before: ",(OrderLots()-_Init_Init_Lot)*MathAbs(OrderOpenPrice()-OrderStopLoss())*100000);
        }
     }

     if(Price > 3)
     {
       for(i=0; i<OrdersTotal(); i++)
       {
          if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
                continue;
          
             if(OrderType() == OP_BUY)
             {
                Price = Bid;
                Color = Blue;
             }
             else
             {
                Price = Ask;
                Color = Red;
             }

             if(OrderType() > 1)
             {
               if(!OrderDelete(OrderTicket()))
                   Print(ErrorDescription(GetLastError()));
             }
             else
             {
               if(!OrderClose(OrderTicket(), OrderLots(), Price, .00002, Color))
                   Print(ErrorDescription(GetLastError()));
             }
       }
     }
*/
// Look thru all the orders
      for(n=0; n<OrdersTotal(); n++)
      {
         if(!OrderSelect(n,SELECT_BY_POS,MODE_TRADES)) // Only those that are active
            continue;
         
         if(OrderSymbol() != Symbol()) // and put onto a chart with our instrument
            continue;

//  Check if it is a Revert, and it's not in market yet
         Ticket = OrderTicket();
         if((OrderMagicNumber() % pool_size != 0) && (OrderType() >1))
         {
//   and its Revertee was closed with Profit
            if(OrderSelect(OrderMagicNumber(),SELECT_BY_TICKET))
            {
                if((OrderProfit() > 0) && (OrderCloseTime() != 0))
                {
//    if it was, then drop the Revert
                     OrderDelete(Ticket);
                     continue;
                }
            }
            OrderSelect(Ticket,SELECT_BY_TICKET);  // restore the focus at the Order
         }
         
//  Check if the Order needs to be reverted (it is in Market)
         if(OrderType() < 2)
         {
// Check that there is no revert yet with this price
            if(OrderType() == OP_BUY)
               rOp = OP_SELLSTOP;
            else
               rOp = OP_BUYSTOP;

            if(No_Revert(rOp, OrderStopLoss()))
            {
               OrderSelect(Ticket,SELECT_BY_TICKET);  // The focus was lost by No_Orders_exactly

// Double the Lot, with  respect of the _Double_First_Lot setting for a first revert
               if((!_Double_First_Lot) && (OrderMagicNumber() % pool_size == 0))
                  rLot = OrderLots();
               else
                  rLot = OrderLots() * 2;

// Limit the lot to the Maximum - reset to the initial once Max is reached
               if(rLot > _Max_Init_Lot)
                  rLot = _Init_Init_Lot; //  * 2

               if(0 > OrderSend(Symbol(), rOp, rLot, OrderStopLoss()
                              , 0       // Slippage
                              , OrderOpenPrice()
                              , NormalizeDouble(2 * OrderStopLoss() - OrderOpenPrice(), Digits)
                              , ""              // Comment
                              , OrderTicket()   // Magic
                              , 0               // Life-Time
                              , Blue))
               {
                  err=GetLastError();
                  Print("error(",err,"): ", ErrorDescription(err));
               }
            }
         }
      }
}
////////////////////////////////////////////////////////   
// Check active orders within $D range
//  Add the $spread value to Range - this makes sence when the channel is tight
//
bool No_Orders_near(int fType, double fPrice)
{
// This receives OP_BUYSTOP (or OP_SELLSTOP), but check OP_BUY (or OP_SELL)
  for(int i=0; i<OrdersTotal(); i++)
  {
    if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
      continue;

    if((OrderSymbol() == Symbol()) && ((fType == OrderType()) || (fType - 2 == OrderType()) || (fType - 4 == OrderType())))
       if(MathAbs(OrderOpenPrice() - fPrice) < Range)
         return (false);
  }
  return (true);
}
////////////////////////////////////////////////////////
// Check active orders exactly at the given price
//
bool No_Revert(int fType, double fPrice)
{
  for(int i=0; i<OrdersTotal(); i++)
  {
    if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
      continue;

    if((OrderSymbol() == Symbol()) && (fType == OrderType()) && (OrderMagicNumber() % pool_size != 0))
       if(OrderOpenPrice() == fPrice)
         return (false);
  }
  return (true);
}
////////////////////////////////////////////////////////
//=   
string ErrorDescription(int fError= -1)
{
  string ps;

  switch(fError)
  {
    case 0: ps=" No error"; break;
    case 1: ps=" No error, but the result is unknown"; break;
    case 2: ps=" Common error"; break;
    case 3: ps=" Invalid parameters"; break;
    case 4: ps=" The server is busy"; break;
    case 5: ps=" Obsolete terminal version"; break;
    case 6: ps=" No connection to the server"; break;
    case 7: ps="  ."; break;
    case 8: ps=" Too frequent requests"; break;
    case 9: ps="     ."; break;

    case 64: ps=" The account is disabled"; break;
    case 65: ps=" Invalid account number"; break;

    case 128: ps=" Operation timeout"; break;
    case 129: ps=" Invalid price"; break;
    case 130: ps=" Invalid SL or TP"; break;
    case 131: ps=" Invalid volume"; break;
    case 132: ps=" Market is closed"; break;
    case 133: ps=" Trading is forbidden"; break;
    case 134: ps=" Not enough money"; break;
    case 135: ps=" Price changed"; break;
    case 136: ps=" No price"; break;
    case 137: ps=" Broker is busy"; break;
    case 138: ps=" New prices"; break;
    case 139: ps=" The order is locked and being processed"; break;
    case 140: ps=" BUY only"; break;
    case 141: ps=" Too many requests"; break;
    case 145: ps="  ,       ."; break;
    case 146: ps="   ."; break;
    case 147: ps="      ."; break;
    case 148: ps="       ,  ."; break;

    case 4000: ps=" No error"; break;
    case 4001: ps="   ."; break;
    case 4002: ps="   -  ."; break;
    case 4003: ps="     ."; break;
    case 4004: ps="     ."; break;
    case 4005: ps="       ."; break;
    case 4006: ps="     ."; break;
    case 4007: ps="     ."; break;
    case 4008: ps="  ."; break;
    case 4009: ps="    ."; break;
    case 4010: ps="     ."; break;
    case 4011: ps=" The string is too long"; break;
    case 4012: ps=" Zero division"; break;
    case 4013: ps=" Zero division"; break;
    case 4014: ps="  ."; break;
    case 4015: ps="  ."; break;
    case 4016: ps="  ."; break;
    case 4017: ps="  DLL  ."; break;
    case 4018: ps="   ."; break;
    case 4019: ps="   ."; break;
    case 4020: ps="      ."; break;
    case 4021: ps="    ,   ."; break;
    case 4022: ps="  ."; break;
    case 4050: ps="    ."; break;
    case 4051: ps="    ."; break;
    case 4052: ps="    ."; break;
    case 4053: ps="  ."; break;
    case 4054: ps="   -."; break;
    case 4055: ps="   ."; break;
    case 4056: ps="  ."; break;
    case 4057: ps="    ."; break;
    case 4058: ps="    ."; break;
    case 4059: ps="      ."; break;
    case 4060: ps="   ."; break;
    case 4061: ps="   ."; break;
    case 4062: ps="    string."; break;
    case 4063: ps="    integer."; break;
    case 4064: ps="    double."; break;
    case 4065: ps="     ."; break;
    case 4066: ps="      ."; break;
    case 4067: ps="     ."; break;
    case 4099: ps=" End of file"; break;

    case 4100: ps="     ."; break;
    case 4101: ps="   ."; break;
    case 4102: ps="    ."; break;
    case 4103: ps="   ."; break;
    case 4104: ps="     ."; break;
    case 4105: ps=" No order selected"; break;
    case 4106: ps=" Unknown symbol"; break;
    case 4107: ps=" Wrong parameter of the trading function"; break;
    case 4108: ps=" Invalid ticket ID"; break;
    case 4109: ps=" Trading is disabled. Enable the expert trading in settings."; break;
    case 4110: ps="    .    ."; break;
    case 4111: ps="    .    ."; break;

    case 4200: ps="   ."; break;
    case 4201: ps="    ."; break;
    case 4202: ps="   ."; break;
    case 4203: ps="   ."; break;
    case 4204: ps="   ."; break;
    case 4205: ps="   ."; break;
    case 4206: ps="    ."; break;
    case 4207: ps="     ."; break;

    default:   ps=" Unknown error";
  }
  return (ps);
}